--=================================================================================
-- Skyline Lua dynamic Scipt - Animal AI ( Evade )
-- 2015 Requires Skyline Version 0.9.4 or greater
--=================================================================================

--[[HeaderImage|aiAnimal_Evade_DynamicProperty_header.png]]

--[[Prop|button]] Update_Values = "Update" 

---------------------------------------------------------------------------------
-- EDITOR: CHARACTER
---------------------------------------------------------------------------------
--[[Prop|Header]]   Character_Size 		= "" 
--[[Prop|float]]	Character_Height 	= 1.0
--[[Prop|float]]	Character_Offset 	= -1.0
--[[Prop|float]]	Character_Width 	= 0.4 

Character_StepSize 		= 0.4;
Character_FallSpeed 	= -9;
Character_SlopeLimit 	= 70;
Character_FlipZ 		= 0;
Character_SkinWidth 	= 0.1;
Character_jumpDownForce	= 40;

---------------------------------------------------------------------------------
-- EDITOR: MOVEMENT
---------------------------------------------------------------------------------
--[[Prop|Header]]   Movement  		= "" 
--[[Prop|float]]	Walk_Speed 		= 2.25
--[[Prop|float]]	Run_Speed 		= 5.0
--[[Prop|float]]	Turning_Speed 	= 200	
--[[Prop|float]]	Walk_Timeout 	= 3000	
--[[Prop|float]]	Run_Timeout 	= 2000	

---------------------------------------------------------------------------------
-- EDITOR: DETECTION
---------------------------------------------------------------------------------
--[[Prop|Header]]   Detection  		= "" 
--[[Prop|float]]	Sensor_Radius 	= 8		
--[[Prop|float]]	Detection_Range = 7		
--[[Prop|float]]	Run_Range 	 	= 5		

---------------------------------------------------------------------------------
-- PRIVATE DATA
---------------------------------------------------------------------------------
ox,oy,oz 					= 0;
obj 						= 0;
walkSpeed 					= Walk_Speed;
turnSpd 					= 0;
attacking 					= 0;
dccID						= 0;
move 						= 0;
turnVal						= 0;
moveVal 					= 0;
running						= false;
moveSpeed 					= 0;
CURRENT_ANIM 				= "Idle1";
NON_ATTACK_ANIM 			= CURRENT_ANIM;
doDamage					= false;
takingDamge					= false;
playerKO 					= 0;

distanceToPredator 			= 0;
predatorID 					= -1;
interval_Walk 				= 3000;
interval_Run 				= 2000;
TIMER_WALK 					= 1;
TIMER_RUN 					= 2; --Interval animal will keep running after predator getting too close
run_range					= 5
detection_range				= 7
sensor_range				= 8

function setupDefaults()
	move 					= 0;
	turnVal 				= 0;
	running					= false;
	CURRENT_ANIM 			= IDLE_1;
	distanceToPredator 		= 99999;
	predatorID 				= -1;
end
---------------------------------------------------------------------------------
-- SETUP DATA
---------------------------------------------------------------------------------
function setupGame(id)
	setupDefaults();
	setup_DynamicValues(id);
	setup_Capsule(id);	
	setup_Anim(id);
	moveSpeed = Walk_Speed; 
	playerKO = 0;
	sensorID = action.getActionID(id, "AI Sensor")
	action.setActionParam(id, sensorID, "AIS Bounds Radius", sensor_range);	
end

function setup_DynamicValues( id )
	Character_Height	= tonumber(	entity.getDynamicProperty(id, "Character_Height"));
	Character_Offset	= tonumber(	entity.getDynamicProperty(id, "Character_Offset"));
	Character_Width		= tonumber(	entity.getDynamicProperty(id, "Character_Width"));
	
	Walk_Speed 			= tonumber(	entity.getDynamicProperty(id, "Walk_Speed"));
	runSpeed			= tonumber(	entity.getDynamicProperty(id, "Run_Speed"));
	turnSpeed			= tonumber(	entity.getDynamicProperty(id, "Turning_Speed"));
	
	run_range			= tonumber(	entity.getDynamicProperty(id, "Run_Range"));
	detection_range		= tonumber(	entity.getDynamicProperty(id, "Detection_Range"));
	sensor_range		= tonumber(	entity.getDynamicProperty(id, "Sensor_Radius"));
	if(sensor_range<=0)then sensor_range=1 end
	
	interval_Walk 		= tonumber(	entity.getDynamicProperty(id, "Walk_Timeout"));
	interval_Run 		= tonumber(	entity.getDynamicProperty(id, "Run_Timeout")); 
end

function setup_Anim(id)
	IDLE_1 		= anim.getFromMap(id,"Idle1");
	WALK 		= anim.getFromMap(id,"Walk");
	RUN			= anim.getFromMap(id,"Run");
	anim.setEnabled(id, WALK,		0);
	anim.setEnabled(id, RUN, 		0);
	anim.setEnabled(id, IDLE_1, 	1);
	
	CURRENT_ANIM 	= IDLE_1;
	NON_ATTACK_ANIM = CURRENT_ANIM;
end

function setup_Capsule(id)	
	if(dccID~=-1)then controller.remove(dccID);	end	
	controller.setData_Offset(0,Character_Offset,0);	
	controller.setData_characterHeight(Character_Height);	
	controller.setData_shapeType(0);	
	controller.setData_characterWidth(Character_Width);	
	controller.setData_stepOffset(Character_StepSize); 	
	controller.setData_gravityY(Character_FallSpeed);	
	controller.setData_slopeLimit(Character_SlopeLimit);	
	controller.setData_flipDirection(Character_FlipZ); 	
	controller.setData_skinWidth(Character_SkinWidth);	
	controller.setData_upDirectionIndex(1);		
	dccID = controller.spawn(obj);	
	controller.setJumpDownforce(dccID,Character_jumpDownForce);	
	-- Ensure the character is above the ground, so no falling thro	
	controller.setPosition(dccID,wx,wy+Character_Height*0.5+0.5,wz)
	controller.setQueryFlag(dccID,3)
end


---------------------------------------------------------------------------------
-- EVENTS
---------------------------------------------------------------------------------
function onInit(objID)
	obj 		= objID;
	dccID 	= -1;
	wx,wy,wz = entity.getPosition(obj);
	CURRENT_ANIM 	= IDLE_1;
	setupGame(obj);
end

function postInit()
	CURRENT_ANIM 	= IDLE_1;
	anim.playAnimation(obj, CURRENT_ANIM, 30, 1);
	moveSpeed = Walk_Speed; 
end

function onUpdate( td )
	moveSpd = 0;
	if(playerKO == 1) then return; end	
	wx,wy,wz = entity.getPosition(obj);
	evade();	
	if(move ==  1) 		then moveSpd =  moveSpeed;  end
	if(move == -1) 		then moveSpd = -moveSpeed; end
	anim.playAnimation(obj, CURRENT_ANIM, 30, 1); 
	entity.turn(obj, 0, turnVal*td , 0);
	controller.move(dccID, moveSpd ); 
end

function onStop(  )
	anim.playAnimation(obj, IDLE_1, 30, 0);
end

function evade()
	chase_evade_Flag = -1; --   1=chase -1=evade
	if(predatorID >0)then
		distanceToPredator = entity.getDistance(predatorID,obj);
		if( distanceToPredator <= detection_range )then 
			doMove( true,1 ) 
			turnBoost = 1;
			if(distanceToPredator <= run_range)then 
				doRun(true);
				time.stopMultiTimer(TIMER_WALK, obj);
				time.startMultiTimer( TIMER_RUN, obj, interval_Run );
				if(distanceToPredator <= 1) then turnBoost = 5; end
			else 
				time.startMultiTimer( TIMER_WALK, obj, interval_Walk );
			end
			tx,ty,tz = entity.inverseTransform(obj, entity.getPosition(predatorID));
			turnVal = turnBoost*50*tx*chase_evade_Flag;
		else
			turnVal = 0;
		end
	end
end


---------------------------------------------------------------------------------
-- SENSOR
---------------------------------------------------------------------------------
-- Players dame field of view. If any npc is in range then dish out damage.
function onSensorContact_Stay( testTable )
	for k,targetID in ipairs( testTable ) do
		predatorID = targetID
	end
end


---------------------------------------------------------------------------------
-- MOVEMENT
---------------------------------------------------------------------------------
function doMove(state,flag)

	if(state == true) then
		if(running==false ) then 
			CURRENT_ANIM 	= WALK; 
		else 
			CURRENT_ANIM 	= RUN; 
		end
		move = 1*flag;
	else
		move = 0; 
		CURRENT_ANIM = IDLE_1;	
	end
	NON_ATTACK_ANIM = CURRENT_ANIM	
end

function doRun(state)
	if(playerKO==1)then return; end
	if(state == true) then
		moveSpeed = runSpeed 
		if(move==1)then CURRENT_ANIM = RUN;end
		running = true;
	else
		if(move==1)then CURRENT_ANIM = WALK;end
		moveSpeed = Walk_Speed; 
		running = false;
	end
	NON_ATTACK_ANIM = CURRENT_ANIM	
end

function doTurn(val)
	if(playerKO==1)then return; end
	turnVal = val ;
	if(turnVal==0)then
		if(move==0)then doMove(false,0); end
	else 
		if(move==0)then CURRENT_ANIM = WALK; end
	end
end


---------------------------------------------------------------------------------
-- TIMER
---------------------------------------------------------------------------------
function onMultiTimer(timerID)
	------------------------------------------------------------------------
	----- Walk Timer
	----- the interval for the animal to stop walking after being chased.
	if(timerID == TIMER_WALK)then
		time.stopMultiTimer(TIMER_WALK, obj);
		doMove(false,1) ;
		turnVal = 0;
	end
	
	------------------------------------------------------------------------
	----- Run Timer
	----- the interval for the animal to stop running.
	if(timerID == TIMER_RUN)then
		time.stopMultiTimer(TIMER_RUN, obj);
		time.startMultiTimer( TIMER_WALK, obj, interval_Walk );
		doRun(false);
		doMove(true,1) ;
		turnVal = 0;
	end
end
