--=================================================================================
-- Skyline Lua dynamic Scipt - FPS NPC Zombie
-- 2015 Requires Skyline Version 0.9.4 or greater
--=================================================================================

--[[HeaderImage|Character_NPC_Zombie_DynamicProperty_header.png]]

---------------------------------------------------------------------------------
-- EDITOR: CHARACTER SETUP
---------------------------------------------------------------------------------
--[[Prop|Header]]   Character_Setup		= "" 
--[[Prop|float]]	Character_Height 	= 1.0
--[[Prop|float]]	Character_Offset 	= -1.0
--[[Prop|float]]	Character_Width 	= 0.4 

Character_StepSize 		= 0.4;
Character_FallSpeed 	= -9;
Character_SlopeLimit 	= 70;
Character_FlipZ 		= 0;
Character_SkinWidth 	= 0.1;
Character_jumpDownForce	= 40;


---------------------------------------------------------------------------------
-- EDITOR: MOVEMENT
---------------------------------------------------------------------------------
--[[Prop|Header]]   Movement  			= "" 
--[[Prop|float]]	Walk_Speed 			= 1.0
--[[Prop|float]]	Run_Speed 			= 4.0
--[[Prop|float]]	Turning_Speed 		= 200	
--[[Prop|float]]	Walk_Timeout 		= 2000	
--[[Prop|float]]	Run_Timeout 		= 1000	

Walk_Speed 			= 2.25
runSpeed			= 5.0
turnSpeed	        = 200
interval_Walk 		= 3000;
interval_Run 		= 2000;

---------------------------------------------------------------------------------
-- EDITOR: DETECTION
---------------------------------------------------------------------------------
--[[Prop|Header]]   Detection  			= "" 
--[[Prop|int]]		Character_Flag		= 1
--[[Prop|string]]	Prey_Flag			= "2"
--[[Prop|float]]	Sensor_Radius 		= 10		
--[[Prop|float]]	Detection_Range 	= 9		
--[[Prop|float]]	Run_Range 	 		= 6		
--[[Prop|float]]	Attack_Range 		= 2.0		

queryFlag 		= 1;
preyFlag 		= "2";
sensor_range	= 10
detection_range	= 9
run_range		= 6
attack_range	= 2.0

---------------------------------------------------------------------------------
-- EDITOR: ANIMATION EXTRA
---------------------------------------------------------------------------------
--[[Prop|Header]]   Action_Timing		= "" ;
--[[Prop|int]] 		Attack1_AnimLength	= 1000;
--[[Prop|int]] 		Attack1_DamageDelay	= 400;	
--[[Prop|int]] 		Attack1_SoundDelay	= 200;	
--[[Prop|float]]	Respawn_Interval	= 5000
--[[Prop|float]]	TakeHit_AnimLength	= 400	
--[[Prop|float]] 	Global_Anim_Speed	= 1.5;	

interval_Respawn 	= 1000;
TakeHit_interval	= 400;

---------------------------------------------------------------------------------
-- EDITOR: ZOMBIE STATS
---------------------------------------------------------------------------------
--[[Prop|Header]]   Zombie_Stats 		= "" ;
--[[Prop|string]] 	Name				= "Zombie"
--[[Prop|float]] 	Max_Health			= 100
--[[Prop|float]] 	Detection_FOV		= 40
--[[Prop|bool]] 	Dangerous			= "true"
--[[Prop|bool]] 	ReSpawn				= "true"

name 			= "Zombie";
maxHealth 		= 100;
fov 			= 40;
isDangerous		= true
respawnEnabled 	= true;

---------------------------------------------------------------------------------
-- EDITOR: ATTACK DAMAGE AMOUNTS
---------------------------------------------------------------------------------
--[[Prop|Header]]   Damage_Amount 		= "" ;
--[[Prop|float]] 	Attack1_Damage		= 10
--[[Prop|float]] 	Attack2_Damage		= 20
--[[Prop|float]] 	Attack3_Damage		= 30
--[[Prop|float]] 	Attack4_Damage		= 40

attack1_damage = 10
attack2_damage = 10
attack3_damage = 10
attack4_damage = 10
 
---------------------------------------------------------------------------------
-- PRIVATE DATA
---------------------------------------------------------------------------------
ox,oy,oz 					= 0;
obj 						= 0;
dccID						= 0;
walkSpeed 					= Walk_Speed;
turnSpd 					= 0;
attacking					= false;
takingDamage 				= false;
move 						= 0;
turnVal						= 0;
moveVal 					= 0;
running						= false;
moveSpeed 					= 0;
CURRENT_ANIM 				= "idle";
NON_ATTACK_ANIM 			= CURRENT_ANIM;
doDamage					= false;
playerKO 					= 0;
outOfAttackRange			= true;
distanceToPredator 			= 0;
prayID 						= -1;
health 						= 100;

TIMER_WALK 					= 1;
TIMER_RUN 					= 2; --Interval animal will keep running after predator getting too close
TIMER_ATTACK1				= 3;
TIMER_ATTACK1_DAMAGEDELAY	= 4;
TIMER_TAKEHIT_1				= 5;
TIMER_RESPAWN 				= 6;

	

---------------------------------------------------------------------------------
-- SETUP DATA
---------------------------------------------------------------------------------
function setupGame(id)
	setupDefaults();
	setup_DynamicValues(id);
	setup_Capsule(id);	
	setup_Anim(id);
	moveSpeed = Walk_Speed; 
	playerKO = 0;
	sensorID = action.getActionID(id, "AI Sensor")
	action.setActionParam(id, sensorID, "AIS Bounds Radius", sensor_range);	
	action.setActionParam(id, sensorID, "AIS Query Data", preyFlag);	
	
end

function setupDefaults()
	move 					= 0;
	turnVal 				= 0;
	running					= false;
	CURRENT_ANIM 			= IDLE_1;
	NON_ATTACK_ANIM			= IDLE_1;
	distanceToPredator 		= 99999;
	prayID 					= -1;
end

function SetDynamicProperty( id, param, value )
	if(param=="Dangerous") then
		if(value=="1")then isDangerous = true; else isDangerous=false; end
	end
	if(param=="ReSpawn") then
		if(value=="1")then respawnEnabled = true; else respawnEnabled=false; end
	end

	if(param=="Character_Height") then Character_Height = value; end
	if(param=="Character_Offset") then Character_Offset = value; end
	if(param=="Character_Width" ) then Character_Width	= value; end
	
	if(param=="Walk_Speed"		) then Walk_Speed 		= value; end
	if(param=="Run_Speed"		) then runSpeed			= value; end
	if(param=="Turning_Speed"	) then turnSpeed	 	= value; end
	if(param=="Walk_Timeout") 		then interval_Walk 	= value; end
	if(param=="Run_Timeout"	) 		then interval_Run 	= value; end
	
	if(param=="Respawn_Interval") 	then interval_Respawn 	= value; end
	
	if(param=="Run_Range"		) then run_range		= value; end
	if(param=="Attack_Range"	) then attack_range		= value; end
	if(param=="Detection_Range"	) then detection_range	= value; end
	if(param=="Sensor_Radius"	) then sensor_range		= value; end
	if(sensor_range<=0)then sensor_range=1 end
	
	if(param=="TakeHit_AnimLength" 	) then TakeHit_interval 	= value; end
	if(param=="Attack1_AnimLength" 	) then attack1_animLength 	= value; end
	if(param=="Attack1_DamageDelay"	) then attack1_damageDelay	= value; end
	if(param=="Attack1_Damage" 		) then attack1_damage		= value; end

	if(param=="Max_Health" 		) then 
		maxHealth = value; 
		health = maxHealth; 
		damage_actionID = action.getActionID(id, "Damage System")
		action.setActionParam(id, damage_actionID, "DS Health", health);
	end
end

function setup_DynamicValues( id )
	Character_Height	= tonumber(	entity.getDynamicProperty(id, "Character_Height"));
	Character_Offset	= tonumber(	entity.getDynamicProperty(id, "Character_Offset"));
	Character_Width		= tonumber(	entity.getDynamicProperty(id, "Character_Width" ));
	
	Walk_Speed 			= tonumber(	entity.getDynamicProperty(id, "Walk_Speed"		));
	runSpeed			= tonumber(	entity.getDynamicProperty(id, "Run_Speed"		));
	turnSpeed			= tonumber(	entity.getDynamicProperty(id, "Turning_Speed"	));
	interval_Walk 		= tonumber(	entity.getDynamicProperty(id, "Walk_Timeout"	));
	interval_Run 		= tonumber(	entity.getDynamicProperty(id, "Run_Timeout"		)); 
	
	interval_Respawn 	= tonumber(	entity.getDynamicProperty(id, "Respawn_Interval")); 
	
	queryFlag			= tonumber(	entity.getDynamicProperty(id, "Character_Flag"	));
	preyFlag			= entity.getDynamicProperty(id, "Prey_Flag"	);
	run_range			= tonumber(	entity.getDynamicProperty(id, "Run_Range"		));
	attack_range		= tonumber(	entity.getDynamicProperty(id, "Attack_Range"	));
	detection_range		= tonumber(	entity.getDynamicProperty(id, "Detection_Range"	));
	sensor_range		= tonumber(	entity.getDynamicProperty(id, "Sensor_Radius"	));
	if(sensor_range<=0)then sensor_range=1 end
	
	local temp1 		= 			entity.getDynamicProperty(id, "Dangerous"		); 
	if(temp1=="1")then isDangerous = true; else isDangerous=false; end
	
	local temp2 		= 			entity.getDynamicProperty(id, "ReSpawn"			); 
	if(temp2=="1")then respawnEnabled = true; else respawnEnabled=false; end
	
	TakeHit_interval 	= tonumber(entity.getDynamicProperty(id, "TakeHit_AnimLength" 	));
	attack1_animLength 	= tonumber(entity.getDynamicProperty(id, "Attack1_AnimLength" 	));
	attack1_damageDelay = tonumber(entity.getDynamicProperty(id, "Attack1_DamageDelay"	));
	attack1_damage		= tonumber(entity.getDynamicProperty(id, "Attack1_Damage" 		));
	
	name				= entity.getDynamicProperty(id, "Name" 		);
	maxHealth			= tonumber(entity.getDynamicProperty(id, "Max_Health" 		));
	health = maxHealth;
	damage_actionID = action.getActionID(id, "Damage System")
	action.setActionParam(id, damage_actionID, "DS Health", health);
	
end

function setup_Anim(id)
	IDLE_1 		= anim.getFromMap(id,"Idle1");
	WALK 		= anim.getFromMap(id,"Walk");
	RUN			= anim.getFromMap(id,"Run");
	ATTACK_1	= anim.getFromMap(id,"Attack1");
	TAKEHIT_1	= anim.getFromMap(id,"TakeHit_1");
	KO_1		= anim.getFromMap(id,"Ko1");
	
	anim.setEnabled(id, WALK,		0);
	anim.setEnabled(id, RUN, 		0);
	anim.setEnabled(id, ATTACK_1, 	0);
	anim.setEnabled(id, TAKEHIT_1, 	0);
	anim.setEnabled(id, KO_1, 		0);
	anim.setEnabled(id, IDLE_1, 	1);
	CURRENT_ANIM 	= IDLE_1;
	NON_ATTACK_ANIM = CURRENT_ANIM;
end

function setup_Capsule(id)	
	if(dccID~=-1)then controller.remove(dccID);	end	
	controller.setData_Offset(0,Character_Offset,0);	
	controller.setData_characterHeight(Character_Height);	
	controller.setData_shapeType(0);	
	controller.setData_characterWidth(Character_Width);	
	controller.setData_stepOffset(Character_StepSize); 	
	controller.setData_gravityY(Character_FallSpeed);	
	controller.setData_slopeLimit(Character_SlopeLimit);	
	controller.setData_flipDirection(Character_FlipZ); 	
	controller.setData_skinWidth(Character_SkinWidth);	
	controller.setData_upDirectionIndex(1);		
	dccID = controller.spawn(obj);	
	controller.setJumpDownforce(dccID,Character_jumpDownForce);	
	-- Ensure the character is above the ground, so no falling thro	
	controller.setPosition(dccID,wx,wy+Character_Height*0.5+0.5,wz)
	controller.setQueryFlag(dccID,queryFlag)
end


---------------------------------------------------------------------------------
-- EVENTS
---------------------------------------------------------------------------------
function onInit(objID)
	obj 			= objID;
	dccID 			= -1;
	wx,wy,wz 		= entity.getPosition(obj);
	CURRENT_ANIM	= IDLE_1;
	setupGame(obj);
end

function postInit()
	moveSpeed 		= Walk_Speed; 
	health 			= maxHealth;
	ox,oy,oz 		= entity.getPosition(obj);
	CURRENT_ANIM 	= IDLE_1;
	anim.playAnimation(obj, CURRENT_ANIM, 30, 1);
	doRun(false);
	doMove( false,1 )
end

function onUpdate( td )
	moveSpd = 0;
	if(playerKO == 1) then return; end	
	wx,wy,wz = entity.getPosition(obj);
	chase_n_evade();
	if(attacking==false and takingDamage==false) then	
		if(move ==  1) 		then moveSpd =  moveSpeed;  end
		if(move == -1) 		then moveSpd = -moveSpeed; end
		anim.playAnimation(obj, CURRENT_ANIM, 30, 1); 
	end
	entity.turn(obj, 0, turnVal*td , 0);
	controller.move(dccID, moveSpd ); 
end

function onStop(  )
	anim.playAnimation(obj, IDLE_1, 30, 0);
end

function chase_n_evade()
	chase_evade_Flag = 1; --   1=chase -1=evade
	if(prayID >0)then
		if(game.getHealth(prayID)<=0)then 
			--prey ko so zombie to wander somewhere
			tx,ty,tz = entity.inverseTransform(obj, ox,oy,oz);
			turnVal = turnSpeed*tx*chase_evade_Flag;
			doMove( true,1 );
			return;
		end
	
		distanceToPredator = entity.getDistance(prayID,obj);
		
		if( distanceToPredator <= detection_range )then 
			doMove( true,1 ) 
			turnBoost = 1;
			
			if(distanceToPredator <= run_range)then 
				doRun(true);
				time.stopMultiTimer( TIMER_WALK, obj);
				time.startMultiTimer( TIMER_RUN, obj, interval_Run );
				
				if(distanceToPredator <= attack_range) then --attack_range
					turnBoost = 10; 
					doMove( false,1 )
					--if(attacking == false) then doAttack(); end
					if( attacking == false and isDangerous == true)then doAttack(); end
				else
					--attacking = false
					outOfAttackRange = true;
				end
				
			else 
				time.startMultiTimer( TIMER_WALK, obj, interval_Walk );
			end
			
			tx,ty,tz = entity.inverseTransform(obj, entity.getPosition(prayID));
			turnVal = turnBoost*turnSpeed*tx*chase_evade_Flag;
		else
			turnVal = 0;
		end
	else
		CURRENT_ANIM			= IDLE_1;
		--NON_ATTACK_ANIM			= IDLE_1;
		--doMove( true,1 );
	end
end


---------------------------------------------------------------------------------
-- SENSOR
---------------------------------------------------------------------------------
-- Players dame field of view. If any npc is in range then dish out damage.
function onSensorContact_Stay( testTable )
	if(playerKO==1)then return; end
	for k,targetID in ipairs( testTable ) do
		prayID = targetID
	end
	--sky.lprint(game.getHealth(prayID))
	--if(game.getHealth(prayID)<=0)then prayID=-1; end
end



---------------------------------------------------------------------------------
-- MOVEMENT
---------------------------------------------------------------------------------
function doMove(state,flag)

	if(state == true) then
		if(running==false ) then 
			CURRENT_ANIM 	= WALK; 
		else 
			CURRENT_ANIM 	= RUN; 
		end
		move = 1*flag;
	else
		move = 0; 
		CURRENT_ANIM = IDLE_1;	
	end
	NON_ATTACK_ANIM = CURRENT_ANIM	
end

function doRun(state)
	if(playerKO==1)then return; end
	if(state == true) then
		moveSpeed = runSpeed 
		if(move==1)then CURRENT_ANIM = RUN;end
		running = true;
	else
		if(move==1)then CURRENT_ANIM = WALK;end
		moveSpeed = Walk_Speed; 
		running = false;
	end
	NON_ATTACK_ANIM = CURRENT_ANIM	
end

function doTurn(val)
	if(playerKO==1)then return; end
	turnVal = val ;
	if(turnVal==0)then
		if(move==0)then doMove(false,0); end
	else 
		if(move==0)then CURRENT_ANIM = WALK; end
	end
end



---------------------------------------------------------------------------------
-- TIMER
---------------------------------------------------------------------------------
function onMultiTimer(timerID)
	------------------------------------------------------------------------
	----- Walk Timer
	----- the interval for the animal to stop walking after being chased.
	if(timerID == TIMER_WALK)then
		time.stopMultiTimer(TIMER_WALK, obj);
		doMove(false,1) ;
		turnVal = 0;
	end
	
	------------------------------------------------------------------------
	----- Run Timer
	----- the interval for the animal to stop running.
	if(timerID == TIMER_RUN)then
		time.stopMultiTimer(TIMER_RUN, obj);
		time.startMultiTimer( TIMER_WALK, obj, interval_Walk );
		doRun(false);
		doMove(true,1) ;
		turnVal = 0;
	end

	------------------------------------------------------------------------
	----- Re-Spawn Timer
	----- the interval for the animal to stop running.
	if(timerID == TIMER_RESPAWN)then
		time.stopMultiTimer(TIMER_RESPAWN, obj);
		doRespawn();
	end

	------------------------------------------------------------------------
	----- Attack Timeout
	----- Timing intervals for attacks
	if(timerID == TIMER_ATTACK1)then
		time.stopMultiTimer(TIMER_ATTACK1, obj);
		time.stopMultiTimer(TIMER_ATTACK1_DAMAGEDELAY, obj);
		attacking 		= false;	
		CURRENT_ANIM 	= NON_ATTACK_ANIM;
	end
	
	if(timerID == TIMER_ATTACK1_DAMAGEDELAY)then
		time.stopMultiTimer(TIMER_ATTACK1_DAMAGEDELAY, obj);
		damageAmt = 10;
		if(outOfAttackRange == false)then
			game.doDamage(prayID,0,0,0, attack1_damage,100);
			if(game.getHealth(prayID)<=0)then --prey KO!
				--sky.print("Prey KO! Zombie wins")
				CURRENT_ANIM = NON_ATTACK_ANIM;
				attacking = false;	
			end
		end
	end

	------------------------------------------------------------------------
	----- Take a Hit timeout
	-----
	if(timerID == TIMER_TAKEHIT_1)then
		time.stopMultiTimer(TIMER_TAKEHIT_1, obj);
		CURRENT_ANIM = NON_ATTACK_ANIM;
		if(playerKO==1)then return; end
		takingDamage = false;
		--sky.trace()
	end
	
end


---------------------------------------------------------------------------------
-- ACTION - Stuff the character needs to perform in game
---------------------------------------------------------------------------------
function doAttack()
	if(playerKO==1)then return; end
	if(game.getHealth(prayID)<=0)then 
		--CURRENT_ANIM =  "zombie_biting";
		return; 
	end
	if(takingDamage==true)then return; end
	--sky.print("doAttack: ")
	outOfAttackRange = false
	CURRENT_ANIM = ATTACK_1;
	anim.setTime(obj, CURRENT_ANIM, 0);
	anim.playAnimation(obj, CURRENT_ANIM, 30, 1);
	attacking 		= true;
	time.startMultiTimer( TIMER_ATTACK1, obj, attack1_animLength );
	time.startMultiTimer( TIMER_ATTACK1_DAMAGEDELAY,obj, attack1_damageDelay );
end

function doRespawn()
	if(respawnEnabled == false)then return; end
	moveSpeed 				= Walk_Speed; 
	health 					= maxHealth;
	move 					= 0;
	turnVal 				= 0;
	running					= false;
	distanceToPredator 		= 99999;
	prayID 					= -1;	
	takingDamage 			= false;
	attacking 				= false;	
	CURRENT_ANIM 			= IDLE_1;
	anim.playAnimation(obj, CURRENT_ANIM, 30, 1);
	playerKO = 0;
	controller.setPosition(dccID, ox,oy,oz);
	--wx,wy,wz = entity.getPosition(obj);
	--sound.play3DSound( AUDIO_RESPAWN, wx,wy,wz);
	--sky.print("RE-SPAWN!!")	
	-- RESET DAMAGE SYSTEM
	damage_actionID = action.getActionID(obj, "Damage System")
	action.setActionParam(obj, damage_actionID, "DS Health", maxHealth);
end

---------------------------------------------------------------------------------
-- DAMAGE
---------------------------------------------------------------------------------
function ds_onDamage( preyhealth, damage,attackerID )
--sky.print(attackerID)
	if(playerKO==1)then return; end
	doRun(true);
	--sky.print("Zombie - "..name.."   Health: "..preyhealth.."   Damage: "..damage,1);
	takingDamage 	= true;
	attacking		= false;
	NON_ATTACK_ANIM = CURRENT_ANIM;
	CURRENT_ANIM = TAKEHIT_1;
	anim.setTime(obj, CURRENT_ANIM, 0);
	anim.playAnimation(obj, CURRENT_ANIM, 30, 0);
	--wx,wy,wz = entity.getPosition(obj);
	--sound.play3DSound( AUDIO_SLASHSOUND, wx,wy,wz);
	time.stopMultiTimer(TIMER_ATTACK1, obj);
	time.stopMultiTimer(TIMER_ATTACK1_DAMAGEDELAY, obj);
	time.startMultiTimer(TIMER_TAKEHIT_1,obj,TakeHit_interval); 
end

function ds_onKo()
	if(playerKO==1)then return; end
	time.stopMultiTimer(TIMER_ATTACK1, obj);
	time.stopMultiTimer(TIMER_ATTACK1_DAMAGEDELAY, obj);
	time.stopMultiTimer(TIMER_TAKEHIT_1, obj);
	--wx,wy,wz = entity.getPosition(obj);
	--sound.play3DSound( AUDIO_KO, wx,wy,wz);
	controller.move(dccID, 0 );
	anim.setLooped(obj, KO_1, 0);
	anim.playAnimation(obj, KO_1, 30, 0);
	playerKO = 1;
	time.startMultiTimer( TIMER_RESPAWN, obj, interval_Respawn );
	
end